# Load the schema and resource classes
from .add import settings
from .allocate import settings
from .deallocate import settings
from .erase_basic import settings
from .erase_sectors import settings
from .generic_settings import *
from .locate import settings
from .receive import settings
from .register import settings
from .remove import settings
from .snapshot import settings
from .test_hard_drive import settings

