from ereuse_devicehub.resources.device.component.settings import Component, ComponentSubSettings


class OpticalDrive(Component):
    pass


class OpticalDriveSettings(ComponentSubSettings):
    _schema = OpticalDrive
