class Component:
    @staticmethod
    def get_types_of_components() -> ():
        return 'GraphicCard', 'HardDrive', 'Motherboard', 'NetworkAdapter', 'Processor', 'RamModule', 'SoundCard', 'OpticalDrive'
