from .graphic_card import settings
from .hard_drive import settings
from .motherboard import settings
from .network_adapter import settings
from .optical_drive import settings
from .processor import settings
from .ram_module import settings
from .sound_card import settings
